from setuptools import setup, find_packages

setup(
    name='SGrSpecificationPythontks4r',
    version='0.1.7',
    packages=find_packages(),
    url='https://github.com/robin-schoch/SGrPython',
    license='',
    author='robin',
    author_email='',
    description='',
    long_description='',
    long_description_content_type='text/plain',
)
