from enum import Enum


class DataTypes(Enum):
    INT = 'INT',
    FLOAT = 'FLOAT',
    STRING = 'STRING',
    ENUM = 'ENUM',
    BOOLEAN = 'BOOLEAN',
    BITMAP = 'BITMAP',
    DATE_TIME = 'DATE_TIME',
