# nothing here
from sgrspecification import (
    communicator,
    functional_profile,
    generic,
    product
)

__all__ = [
    "communicator",
    "functional_profile",
    "generic",
    "product"
]
